function isAllowedMark(code) {
    return AllowedMarks[code] === true;
}
function isPrimary(code) {
    return PrimaryCharacters[code] !== undefined;
}
function isChar(grapheme, primary, diacritic) {
    return grapheme.primary === primary && grapheme.marks.length === 1 && grapheme.marks[0] === diacritic;
}
function isComb(grapheme1, grapheme2, first, second) {
    return grapheme1.primary == first && grapheme2.primary == second;
}
function calculateGraphemes(id) {
    var graphemes = [];
    let L = id.length;
    if (L == 0)
        return [];
    graphemes.push({ primary: id.charCodeAt(0), marks: [] });
    let firstCode = graphemes[0].primary;
    if (!isPrimary(firstCode)) {
        return undefined;
    }
    for (let i = 1; i < L; i++) {
        let code = id.charCodeAt(i);
        if (isAllowedMark(code)) {
            let g = graphemes[graphemes.length - 1];
            if (g.primary === 0x0020 || g.primary === 0x002D)
                return undefined;
            g.marks.push(code);
        }
        else if (isPrimary(code)) {
            graphemes.push({ primary: code, marks: [] });
        }
        else {
            return undefined;
        }
    }
    return graphemes;
}
function isLetter(code) {
    return code >= 0x0061 && code <= 0x007a;
}
function removeSpaces(id) {
    id = id.trim();
    while (true) {
        var r = id.replace("  ", " ");
        if (r == id)
            return id;
        id = r;
    }
}
function convert(id) {
    id = removeSpaces(id);
    let graphemes = calculateGraphemes(id.normalize("NFD"));
    if (graphemes === undefined) {
        return undefined;
    }
    var result = "";
    let L = graphemes.length;
    var i = 0;
    while (i < L) {
        let g = graphemes[i];
        if (isChar(g, 0x0418, 0x0306) || isChar(g, 0x0438, 0x0306)) {
            result += "j";
            i += 1;
        }
        else if (isChar(g, 0x0406, 0x0308) || isChar(g, 0x0456, 0x0308)) {
            result += "yi";
            i += 1;
        }
        else if (isChar(g, 0x0415, 0x0308) || isChar(g, 0x0435, 0x0308)) {
            result += "yo";
            i += 1;
        }
        else if (i + 1 < L) {
            let h = graphemes[i + 1];
            if (h.primary === 0x03A5 || h.primary === 0x03C5) {
                let p = g.primary;
                if (p === 0x0391 || p === 0x03B1) {
                    result += "au";
                    i += 2;
                }
                else if (p === 0x0395 || p === 0x03B5) {
                    result += "eu";
                    i += 2;
                }
                else if (p == 0x039F || p == 0x03BF) {
                    result += "ou";
                    i += 2;
                }
                else {
                    result += PrimaryCharacters[p];
                    i += 1;
                }
            }
            else {
                result += PrimaryCharacters[g.primary];
                i += 1;
            }
        }
        else {
            result += PrimaryCharacters[g.primary];
            i += 1;
        }
    }
    if (result.length === 0 || !isLetter(result.charCodeAt(0)))
        return undefined;
    if (result[result.length - 1] == "-")
        return undefined;
    if (result.indexOf("--") >= 0)
        return undefined;
    return result;
}
function runConversion() {
    let input = document.getElementById("cosmo-id");
    let message = document.getElementById("message");
    let original = document.getElementById("original-id");
    let simplified = document.getElementById("simplified-id");
    let cosmo = input.value;
    original.innerText = cosmo.replace(/\u0020/g, "\u00a0");
    let simple = convert(cosmo);
    if (simple === undefined) {
        message.innerText = " is not a cosmopolitan identifier.";
        simplified.innerText = "";
    }
    else {
        if (simple != cosmo) {
            message.innerText = " is equivalent to ";
            simplified.innerText = simple;
        }
        else {
            message.innerText = " is already a simple identifier.";
            simplified.innerText = "";
        }
    }
}
function clearConversion() {
    let input = document.getElementById("cosmo-id");
    let message = document.getElementById("message");
    let original = document.getElementById("original-id");
    let simplified = document.getElementById("simplified-id");
    input.value = "";
    message.innerText = "";
    original.innerText = "";
    simplified.innerText = "";
}
